/** 
 @file upnp.js
 
 @brief This file contain implementation of upnp protocol.

 @author: mobile2DS

 @date: 15.12.2013
 */

/**
 * @brief Parse UPNP XML response and return value for specific field
 * 
 * @param szXMLRespond -
 *            respond from upnp device in XML syntax
 * @param szXMLFieldName -
 *            searched field name enclosing the value
 * @param szContainsText -
 *            filter that found value must contain this text. if '' no filter
 *            applying
 * 
 * @return
 */
function upnpParseXMLRespond(szXMLRespond, szXMLFieldName, szContainsText) {
	// lInfo.logln(szXMLRespond);
	var searched_value = '';
	var found = 0;
	var xml_text = '';
	var start_pos = 0;

	xml_text = szXMLRespond;

	do {
		start_pos = xml_text.indexOf('<' + szXMLFieldName);

		if (start_pos == -1) {
			break;
		}

		start_pos += szXMLFieldName.length + 1;
		start_pos += 1 /* skip '>' */;

		var end_pos = xml_text.indexOf('</' + szXMLFieldName);

		if (end_pos == -1) {
			break;
		}
		;

		searched_value = xml_text.substring(start_pos, end_pos);

		if (szContainsText != "") {
			if (searched_value.indexOf(szContainsText) == -1) {
				start_pos = end_pos + szXMLFieldName.length + 1;
				xml_text = xml_text.substring(start_pos, xml_text.length);

	//			lInfo.logln("Filter not match, continue search: : " + xml_text);
			} else {
				found = 1;
			}
		} else {
			found = 1;
		}
	} while ((szContainsText != "") && (found == 0) && (xml_text != ""))

//	lInfo.logln("XML parser found: " + searched_value);

	return searched_value;
}

/*
 * ---------------------------------------------------------------------------
 * UPNP related METHODS
 */

/**
 * @brief Execute UPNP command
 * 
 * @param
 * 
 * @return
 */

function upnpPlay(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#Play"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + "/MediaRenderer/AVTransport/Control";
		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Play xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<Speed>1</Speed>' + '</u:Play>' + '</s:Body>' + '</s:Envelope>'
		var result = h.post(url, '', action);
		// lInfo.logln(JSON.stringify(result));
		if (result != null && result.body != null)
			lInfo.logln(result.body);

	} catch (e) {
		lInfo.logln("Error executing upnpPlay command " + e);
	}
}

function upnpPause(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:"urn:schemas-upnp-org:service:AVTransport:1#Pause"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + "/MediaRenderer/AVTransport/Control";
		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Pause xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:Pause>' + '</s:Body>' + '</s:Envelope>'

		h.post(url, '', action);
	} catch (e) {
		lInfo.logln("Error executing upnpStop command " + e);
	}
}

function upnpStop(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:"urn:schemas-upnp-org:service:AVTransport:1#Stop"';

		var h = new HTTP(oCurlOptions, http_headers);
		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + "/MediaRenderer/AVTransport/Control";
		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Stop xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:Stop>' + '</s:Body>' + '</s:Envelope>'
		h.post(url, '', action);
	} catch (e) {
		lInfo.logln("Error executing upnpStop command " + e);
	}
}

/**
 * @brief Perform get currently played media content src from upnp renderer
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
/*
 * function upnpGetMediaContent(nPlayerId, szIpAddressRenderer, nPortRenderer,
 * szServiceType, oResult, oCurlOptions) { try { var http_headers = [];
 * http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
 * http_headers['extra_hdr2'] = 'SOAPAction: "urn:schemas-upnp-org:service:
 * AVTransport:1#GetMediaInfo"';
 * 
 * var h = new HTTP(oCurlOptions, http_headers);
 * 
 * h.reset();
 * 
 * var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer +
 * szServiceType; lInfo.logln("URL "+url);
 * 
 * var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope
 * s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
 * xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">' + '<s:Body>' + '<u:GetMediaInfo
 * xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetMediaInfo>' + '</s:Body>' + '</s:Envelope>'
 * 
 * var result = h.post(url, '', action); if (result != null ) oResult.status =
 * result.status; oResult.data = upnpParseXMLRespond(result.body, 'CurrentURI',
 * '');
 * 
 * lInfo.logln("Extracted media content: " + oResult.data); } catch (e) {
 * lInfo.logln("Error executing upnpGetMediaContent command " + e); } return ""; }
 */

function upnpGetSonosPlaylist(szIpAddressRenderers, nMaxCount, bOnlyTemporaryQueues, oCurlOptions) {
	var http_headers = [];
	http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
	http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:ContentDirectory:1#Browse"';

	var h = new HTTP(oCurlOptions, http_headers);

	for ( var kIndex = 0; kIndex < szIpAddressRenderers.length; kIndex++) {
		try {
			h.reset();
			var szIpAddressRenderer = szIpAddressRenderers[kIndex];

			var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaServer/ContentDirectory/Control';
			lInfo.logln("URL " + url);

			var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
					+ '<s:Body>' + '    <u:Browse xmlns:u="urn:schemas-upnp-org:service:ContentDirectory:1">' + '    <ObjectID>SQ:</ObjectID>' + '    <BrowseFlag>BrowseDirectChildren</BrowseFlag>'
					+ '    <Filter />' + '    <StartingIndex>0</StartingIndex>' + '    <RequestedCount>' + nMaxCount + '</RequestedCount>' + '    <SortCriteria />' + '    </u:Browse></s:Body>'
					+ '</s:Envelope>'
			lInfo.logln("sonosPlayListAction=" + action);
			var result = h.post(url, '', action);
			if (result != null && result.body != null) {
				lInfo.logln(result.body);
				var arrayLists = new Array();
				var aList = getXMLFields(result.body, 'container');
				if (aList != null) {
					lInfo.logln("Playlist size:" + aList.length);

					for ( var iIndex = 0; iIndex < aList.length; iIndex++) {
						var sData = aList[iIndex].data;

						var sTitle = "";
						var sID = "";
						var sCall = "";

						var matches = sData.match(/<dc:title>(.*?)<\/dc:title>/);
						if (matches != null) {
							sTitle = matches[1];

							matches = sData.match(/<res[^>]*>(.*?)<\/res>/);
							if (matches != null) {
								sCall = matches[1];
							}

							matches = sData.match(/id="(.*?)"/);
							if (matches != null) {
								sID = matches[1];
							}

							if (sID != "" && sCall != "") {

								if ((bOnlyTemporaryQueues && sTitle.indexOf(TMP_SONOS_QUEUE_NAME) > -1) || // we
								// show
								// only
								// tmp
								// queues
								(!bOnlyTemporaryQueues && sTitle.indexOf(TMP_SONOS_QUEUE_NAME) < 0) // we
								// skip
								// tmp
								// queues
								// of
								// addon
								) {
									arrayLists.push({
										title : sTitle,
										listID : sID,
										listCall : sCall
									});
									lInfo.logln(sTitle + " - " + sID + " - " + sCall);
								}
							}
						}
					}

					lInfo.logln("sonosPlayListAction END");
				}
				return arrayLists;
			}
		} catch (e) {
			lInfo.logln("Error executing upnpGetSonosPlaylist command on player " + szIpAddressRenderers[iIndex] + ' : ' + e);
		}
	}
	return null;
}

function upnpGetSonosFavorites(szIpAddressRenderers, nMaxCount, oCurlOptions) {
	var http_headers = [];
	http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
	http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:ContentDirectory:1#Browse"';

	var h = new HTTP(oCurlOptions, http_headers);

	try {
		for ( var kIndex = 0; kIndex < szIpAddressRenderers.length; kIndex++) {
			var szIpAddressRenderer = szIpAddressRenderers[kIndex];

			h.reset();

			var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaServer/ContentDirectory/Control';
			lInfo.logln("URL " + url);

			var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
					+ '<s:Body>' + '    <u:Browse xmlns:u="urn:schemas-upnp-org:service:ContentDirectory:1">' + '    <ObjectID>FV:2</ObjectID>' + '    <BrowseFlag>BrowseDirectChildren</BrowseFlag>'
					+ '    <Filter />' + '    <StartingIndex>0</StartingIndex>' + '    <RequestedCount>' + nMaxCount + '</RequestedCount>' + '    <SortCriteria />' + '    </u:Browse></s:Body>'
					+ '</s:Envelope>'
			lInfo.logln("sonosSonosFavorites=" + action);

			var result = h.post(url, '', action);
			if (result != null && result.body != null) {
				var arrayLists = new Array();

				var aList = getXMLFields(result.body, 'item');
				if (aList != null) {
					lInfo.logln("Favorites size:" + aList.length);

					for ( var iIndex = 0; iIndex < aList.length; iIndex++) {
						var sData = aList[iIndex].data;

						var sTitle = "";
						var sID = "";
						var sCall = "";
						var sDesc = "";
						var sMeta = "";

						var matches = sData.match(/<dc:title>(.*?)<\/dc:title>/);
						if (matches != null) {
							sTitle = matches[1].replace(/&amp;/g, "&").replace(/&lt;/g, "<").replace(/&gt;/g, ">").replace(/&quot;/g, '"').replace(/&#039;/g, "'");
							;

							matches = sData.match(/<r:description>(.*?)<\/r:description>/);
							if (matches != null) {
								sDesc = matches[1].replace(/&amp;/g, "&").replace(/&lt;/g, "<").replace(/&gt;/g, ">").replace(/&quot;/g, '"').replace(/&#039;/g, "'");
								;
							}

							matches = sData.match(/<res[^>]*>(.*?)<\/res>/);
							if (matches != null) {
								sCall = matches[1].replace(/&amp;/g, "&").replace(/&lt;/g, "<").replace(/&gt;/g, ">").replace(/&quot;/g, '"').replace(/&#039;/g, "'");
							}

							matches = sData.match(/<r:resMD[^>]*>(.*?)<\/r:resMD>/);
							if (matches != null) {

								sMeta = matches[1].replace(/&amp;/g, "&").replace(/&lt;/g, "<").replace(/&gt;/g, ">").replace(/&quot;/g, '"').replace(/&#039;/g, "'");
							}

							matches = sData.match(/id="(.*?)"/);
							if (matches != null) {
								sID = matches[1];
							}

							var bIsSingleTrack = (sData.indexOf("object.item.audioItem.musicTrack") > -1);

							if (sID != "" && sCall != "") {
								arrayLists.push({
									title : sTitle,
									description : sDesc,
									listID : sID,
									listCall : sCall,
									metaData : sMeta,
									isSingleTrack : bIsSingleTrack
								});
								lInfo.logln(sTitle + " - " + sID + " - " + sCall);
							}
						}
					}
				}
				return arrayLists;
			}
		}
	} catch (e) {
		lInfo.logln("Error executing upnpGetSonosFavorites command " + e);
	}
	return null;
}

function upnpGetCurrentQueue(szIpAddressRenderer, nMaxResults, oCurlOptions) {
	try {
		lInfo.logln("upnpGetCurrentQueue: " + szIpAddressRenderer);
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:ContentDirectory:1#Browse"';

		var h = new HTTP(oCurlOptions, http_headers);
		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaServer/ContentDirectory/Control';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '  <u:Browse xmlns:u="urn:schemas-upnp-org:service:ContentDirectory:1">' + '    <ObjectID>Q:0</ObjectID>' + '    <BrowseFlag>BrowseDirectChildren</BrowseFlag>'
				+ '    <Filter />' + '    <StartingIndex>0</StartingIndex>' + '    <RequestedCount>' + nMaxResults + '</RequestedCount>' + '    <SortCriteria />' + '  </u:Browse>' + '</s:Body>'
				+ '</s:Envelope>';

		var result = h.post(url, '', action);

		var arrayLists = new Array();
		if (result != null && result.body != null) {
			var aList = getXMLFields(result.body, 'item');
			if (aList != null) {
				lInfo.logln("size:" + aList.length);

				for ( var iIndex = 0; iIndex < aList.length; iIndex++) {
					var sData = aList[iIndex].data;

					var sTitle = "";
					var sID = "";
					var sArtist = "";
					var nTrackPos = -1;
					var sAlbumUri = "";

					var matches = sData.match(/<dc:title>(.*?)<\/dc:title>/);
					if (matches != null) {
						sTitle = matches[1];

						matches = sData.match(/<res[^>]*>(.*?)<\/res>/);
						if (matches != null) {
							sID = matches[1];
						}

						matches = sData.match(/<upnp:albumArtURI>(.*?)<\/upnp:albumArtURI>/);
						if (matches != null) {
							sAlbumUri = matches[1];
						}

						matches = sData.match(/<r:albumArtist>(.*?)<\/r:albumArtist>/);
						if (matches != null)
							sArtist = matches[1];
						else {
							matches = sData.match(/<upnp:album>(.*?)<\/upnp:album>/);
							if (matches != null)
								sArtist = matches[1];
						}

						matches = sData.match(/<upnp:originalTrackNumber>(.*?)<\/upnp:originalTrackNumber>/);
						if (matches != null) {
							nTrackPos = matches[1];
						}

						if (sID != "") {
							arrayLists.push({
								title : htmlUnescape(sTitle),
								playID : unescape(sID),
								// albumUri:unescape(sAlbumUri),
								artist : htmlUnescape(sArtist),
								trackPos : nTrackPos
							});
						}
					}
				}
			}
			// lInfo.logln("media info: " + result.body);
		}
		return arrayLists;

		/*
		 * matches = result.body.match(/<TrackMetaData>(.*?)<\/TrackMetaData>/);
		 * if (matches != null) { var sMetadata = cleanXMLText(matches[1]); var
		 * sUrl = ""; var sArtist = ""; var sTitle = ""; var sLength = ""; var
		 * bRadioStation = true;
		 * 
		 * matches = sMetadata.match(/<res[^>]*>(.*?)<\/res>/); if (matches !=
		 * null) sUrl = matches[1];
		 * 
		 * matches = sMetadata.match(/<dc:title>(.*?)<\/dc:title>/); if
		 * (matches != null) sTitle = matches[1];
		 * 
		 * matches = sMetadata.match(/ duration="(.*?)">/); if (matches != null) { //
		 * mp3 track bRadioStation = false;
		 * 
		 * sLength = matches[1];
		 * 
		 * matches = sMetadata.match(/<r:albumArtist>(.*?)<\/r:albumArtist>/);
		 * if (matches != null) sArtist = matches[1]; } lInfo.logln(sUrl+" * "
		 * +sTitle); }
		 */
	} catch (e) {
		lInfo.logln("Error executing upnpGetCurrentQueue command " + e);
	}
	return null;
}

function upnpGetCurrentRadio(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#GetMediaInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetMediaInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetMediaInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
		if (result != null && result.body != null) {
			matches = result.body.match(/<CurrentURIMetaData>(.*?)<\/CurrentURIMetaData>/);
			if (matches != null) {
				var oReturn = new Object();

				var sMetadata = cleanXMLText(matches[1]);
				var sUrl = "";
				var sTitle = "";

				matches = sMetadata.match(/<dc:title>(.*?)<\/dc:title>/);
				if (matches != null)
					sTitle = matches[1];

				matches = result.body.match(/<CurrentURI>(.*?)<\/CurrentURI>/);
				if (matches != null)
					sUrl = matches[1];

				oReturn.url = sUrl;
				oReturn.title = htmlUnescape(sTitle);
				oReturn.length = "";
				oReturn.artist = "";
				oReturn.isRadio = true;

				lInfo.logln("Found radio: " + sUrl + " * " + sTitle);
				return oReturn;
			}
		}
	} catch (e) {
		lInfo.logln("Error executing upnpGetCurrentRadio command " + e);
	}
	return null;
}

function upnpGetCurrentTrack(szIpAddressRenderer, oCurlOptions) {
	try {
		var posInfo = upnpGetPositionInfo(szIpAddressRenderer, oCurlOptions);
		if (posInfo != null && posInfo != "") {
			var matches = posInfo.match(/<TrackMetaData>(.*?)<\/TrackMetaData>/);
			if (matches != null) {
				var oReturn = new Object();

				var sMetadata = cleanXMLText(matches[1]);
				var sUrl = "";
				var sArtist = "";
				var sTitle = "";
				var sLength = "";
				var bRadioStation = true;

				matches = sMetadata.match(/ duration="(.*?)">/);
				if (matches != null) { // mp3 track
					bRadioStation = false;
					sLength = matches[1];

					matches = sMetadata.match(/<res[^>]*>(.*?)<\/res>/);
					if (matches != null)
						sUrl = matches[1];

					matches = sMetadata.match(/<r:albumArtist>(.*?)<\/r:albumArtist>/);
					if (matches != null)
						sArtist = matches[1];
					else {
						matches = sMetadata.match(/<upnp:album>(.*?)<\/upnp:album>/);
						if (matches != null)
							sArtist = matches[1];
					}

					matches = sMetadata.match(/<dc:title>(.*?)<\/dc:title>/);
					if (matches != null)
						sTitle = matches[1];

					oReturn.url = sUrl;
					oReturn.title = htmlUnescape(sTitle);
					oReturn.length = sLength;
					oReturn.artist = htmlUnescape(sArtist);
					oReturn.isRadio = false;
				} else {
					oReturn = upnpGetCurrentRadio(szIpAddressRenderer, oCurlOptions);
				}

				lInfo.logln("Found track " + oReturn.url + " * " + oReturn.title);
				return oReturn;
			}
		}
	} catch (e) {
		lInfo.logln("Error executing upnpGetCurrentTrack command " + e);
	}
	return null;
}

function upnpSetContentSrc(szIpAddressRenderer, szUpnpServerContent, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#SetAVTransportURI"';

		var h = new HTTP(oCurlOptions, http_headers);
		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:SetAVTransportURI xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<CurrentURI>' + szUpnpServerContent + '</CurrentURI>'
				+ '<CurrentURIMetaData />' + '</u:SetAVTransportURI>' + '</s:Body>' + '</s:Envelope>';

		h.post(url, '', action);
	} catch (e) {
		lInfo.logln("Error executing upnpSetContentSrc command " + e);
	}
}

function upnpSetRadio(szIpAddressRenderer, szUpnpServerContent, szTitle, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#SetAVTransportURI"';

		var h = new HTTP(oCurlOptions, http_headers);
		h.reset();

		szTitle = szTitle.replace('&', '&amp;');

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';
		var sMetaData = '&lt;DIDL-Lite xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:upnp="urn:schemas-upnp-org:metadata-1-0/upnp/" xmlns:r="urn:schemas-rinconnetworks-com:metadata-1-0/" xmlns="urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/"&gt;'
				+ ' &lt;item id="-1" parentID="-1" restricted="true"&gt;'
				+ ' &lt;dc:title&gt;'
				+ szTitle
				+ '&lt;/dc:title&gt;'
				+ ' &lt;upnp:class&gt;object.item.audioItem.audioBroadcast&lt;/upnp:class&gt;'
				+ ' &lt;desc id="cdudn" nameSpace="urn:schemas-rinconnetworks-com:metadata-1-0/"&gt;SA_RINCON65031_&lt;/desc&gt;' + ' &lt;/item&gt;  &lt;/DIDL-Lite&gt;';
		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:SetAVTransportURI xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<CurrentURI>' + szUpnpServerContent + '</CurrentURI>'
				+ '<CurrentURIMetaData>' + sMetaData + '</CurrentURIMetaData>' + '</u:SetAVTransportURI>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
		if (result != null && result.body != null)
			lInfo.logln(result.body);
	} catch (e) {
		lInfo.logln("Error executing upnpSetRadio command " + e);
	}
}

function upnpClearQueue(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#RemoveAllTracksFromQueue"';

		var h = new HTTP(oCurlOptions, http_headers);
		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';
		lInfo.logln(url);

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:RemoveAllTracksFromQueue xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:RemoveAllTracksFromQueue>' + '</s:Body>'
				+ '</s:Envelope>';

		h.post(url, '', action);
	} catch (e) {
		lInfo.logln("Error executing upnpClearQueue command " + e);
	}
}

function upnpLoadPlaylist(szIpAddressRenderer, szPlaylist, oCurlOptions, sMetaData) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#AddURIToQueue"';

		var h = new HTTP(oCurlOptions, http_headers);
		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';

		lInfo.logln("szPlaylist=" + szPlaylist);

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:AddURIToQueue xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<EnqueuedURI>' + szPlaylist + '</EnqueuedURI>';// .replace('&',
		// '&amp;');
		if (sMetaData == null)
			action += '<EnqueuedURIMetaData />';
		else
			action += '<EnqueuedURIMetaData>' + sMetaData.replace(/</g, "&lt;").replace(/>/g, "&gt;") + '</EnqueuedURIMetaData>';
		action += '<DesiredFirstTrackNumberEnqueued>0</DesiredFirstTrackNumberEnqueued>' + '<EnqueueAsNext>1</EnqueueAsNext>' + '</u:AddURIToQueue>' + '</s:Body>' + '</s:Envelope>';

		lInfo.logln(action);
		var result = h.post(url, '', action);
		var sReturn = "";
		if (result != null && result.body != null) {
			sReturn = result.body;
			lInfo.logln("upnpLoadPlaylist=" + result.body);
		}
		return sReturn;
	} catch (e) {
		lInfo.logln("Error executing upnpLoadPlaylist command " + e);
	}
	return "";
}

function upnpSeek(szIpAddressRenderer, bByTrack, trackNr, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#Seek"';

		var h = new HTTP(oCurlOptions, http_headers);
		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';

		var sUnit = 'TRACK_NR';
		if (!bByTrack)
			sUnit = 'REL_TIME';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Seek xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<Unit>' + sUnit + '</Unit>' + '<Target>' + trackNr + '</Target>'
				+ '</u:Seek>' + '</u:Seek>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
		if (result != null && result.body != null)
			lInfo.logln(result.body);
	} catch (e) {
		lInfo.logln("Error executing upnpSeek command " + e);
	}
}

function upnpGetMute(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:GroupRenderingControl:1#GetGroupMute"';

		var h = new HTTP(oCurlOptions, http_headers);
		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/GroupRenderingControl/Control';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetGroupMute xmlns:u="urn:schemas-upnp-org:service:GroupRenderingControl:1">' + '<InstanceID>0</InstanceID>' + '</u:GetGroupMute>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		if (result != null && result.body != null) {
			lInfo.logln(result.body);
			var sMute = upnpParseXMLRespond(result.body, 'CurrentMute', '');
			if (sMute != "" && sMute == "1")
				return true;
		}
	} catch (e) {
		lInfo.logln("Error executing upnpSetMute command " + e);
	}
	return false;
}

function upnpSetMute(szIpAddressRenderer, bMute, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:GroupRenderingControl:1#SetGroupMute"';

		var h = new HTTP(oCurlOptions, http_headers);
		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/GroupRenderingControl/Control';

		var nMute = 0;
		if (bMute)
			nMute = 1;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:SetGroupMute xmlns:u="urn:schemas-upnp-org:service:GroupRenderingControl:1">' + '<InstanceID>0</InstanceID>' + '<DesiredMute>' + nMute + '</DesiredMute>'
				+ '</u:SetGroupMute>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		if (result != null && result.body != null)
			lInfo.logln(result.body);
	} catch (e) {
		lInfo.logln("Error executing upnpSetMute command " + e);
	}
}

function upnpSetVolume(szIpAddressRenderer, nVolume, oCurlOptions) {
	lInfo.logln("upnpSetVolume " + szIpAddressRenderer + " to " + nVolume);
	try {
		if (nVolume != null) {
			var http_headers = [];
			http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
			http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:RenderingControl:1#SetVolume"';

			var h = new HTTP(oCurlOptions, http_headers);
			h.reset();

			var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/RenderingControl/Control';

			var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
					+ '<s:Body>' + '<u:SetVolume xmlns:u="urn:schemas-upnp-org:service:RenderingControl:1">' + '<InstanceID>0</InstanceID>' + '<Channel>Master</Channel>' + '<DesiredVolume>' + nVolume
					+ '</DesiredVolume>' + '</u:SetVolume>' + '</s:Body>' + '</s:Envelope>';

			var result = h.post(url, '', action);

			if (result != null && result.body != null)
				lInfo.logln(result.body);
		}
	} catch (e) {
		lInfo.logln("Error executing upnpSetVolume command " + e);
	}
}

function upnpGetTransportInfo(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#GetTransportInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetTransportInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetTransportInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
		if (result != null && result.body != null)
			return upnpParseXMLRespond(result.body, 'CurrentTransportState', '')
	} catch (e) {
		lWarn.logln("Error executing upnpGetTransportInfo command " + e);
	}
	return "";
}

function upnpGetPositionInfo(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#GetPositionInfo"';//

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetPositionInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetPositionInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
		if (result != null && result.body != null) {
			return result.body;
		}
	} catch (e) {
		lWarn.logln("Error executing upnpGetPositionInfo command " + e);
	}
	return "";
}

function upnpGetMediaInfo(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#GetMediaInfo"';//

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();
		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetMediaInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetMediaInfo>' + '</s:Body>' + '</s:Envelope>';
		result = h.post(url, '', action);
		if (result != null && result.body != null)
			return result.body;
	} catch (e) {
		lWarn.logln("Error executing upnpGetMediaInfo command " + e);
	}
	return null;
}

function upnpGetVolume(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:RenderingControl:1#GetVolume"';//

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();
		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/RenderingControl/Control';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetVolume xmlns:u="urn:schemas-upnp-org:service:RenderingControl:1">' + '<InstanceID>0</InstanceID>' + '<Channel>Master</Channel>' + '</u:GetVolume>' + '</s:Body>'
				+ '</s:Envelope>';
		var result = h.post(url, '', action);
		if (result != null && result.body != null) {
			return upnpParseXMLRespond(result.body, 'CurrentVolume', '')
		}
	} catch (e) {
		lWarn.logln("Error executing upnpGetVolume command " + e);
	}
	return null;
}

/*
 * sModeName: NORMAL, REPEAT_ALL, SHUFFLE_NOREPEAT, SHUFFLE
 */
function upnpSetPlayMode(szIpAddressRenderer, sModeName, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#SetPlayMode"';//

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();
		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:SetPlayMode xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<NewPlayMode>' + sModeName + '</NewPlayMode>'
				+ '</u:SetPlayMode>' + '</s:Body>' + '</s:Envelope>';
		h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpSetPlayMode command " + e);
	}
}

function upnpGetPlayMode(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#GetTransportSettings"';//

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();
		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + ' <u:GetTransportSettings xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + ' <InstanceID>0</InstanceID>' + ' </u:GetTransportSettings>' + '</s:Body>'
				+ '</s:Envelope>';
		var result = h.post(url, '', action);
		if (result != null && result.body != null) {
			return upnpParseXMLRespond(result.body, 'PlayMode', '')
		}
	} catch (e) {
		lWarn.logln("Error executing upnpSetPlayMode command " + e);
	}
	return "";
}

function upnpGetSonosGroups(szIpAddressRenderers, oCurlOptions) {
	var http_headers = [];
	http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
	http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:ZoneGroupTopology:1#GetZoneGroupState"';//

	var h = new HTTP(oCurlOptions, http_headers);
	for ( var kIndex = 0; kIndex < szIpAddressRenderers.length; kIndex++) {
		try {
			h.reset();
			var szIpAddressRenderer = szIpAddressRenderers[kIndex];
			var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/ZoneGroupTopology/Control';
	
			var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
					+ '<s:Body>' + ' <u:GetZoneGroupState xmlns:u="urn:schemas-upnp-org:service:ZoneGroupTopology:1" />' + '</s:Body>' + '</s:Envelope>';
			var result = h.post(url, '', action);
			var arrayGroups = new Array();
			if (result != null && result.body != null) {
				var aGroups = getXMLFields(result.body, 'ZoneGroup', '');
				if (aGroups != null && aGroups.length > 0) {
					for ( var iIndex = 0; iIndex < aGroups.length; iIndex++) {
						aGroups[iIndex].members = null;
						lInfo.logln('GROUPING INFO ' + iIndex + ' data ' + aGroups[iIndex].data);
						var fInvisibile=true;
						var aAllPlayesInGroup = getXMLFields(result.body, 'ZoneGroupMember', '/>');
						var arrayMembers = new Array();
						for ( var yIndex = 0; yIndex < aAllPlayesInGroup.length; yIndex++) {
							arrayMembers.push(aAllPlayesInGroup[yIndex].data);
							lInfo.logln('GROUPING SUB INFO ' + yIndex + ' data ' + aAllPlayesInGroup[yIndex].data);
						}
						var aAllPlayesInSubGroup = getXMLFields(aGroups[iIndex].data, 'ZoneGroupMember', '/>');
						
						for ( var yIndex = 0; yIndex < aAllPlayesInSubGroup.length; yIndex++) {
							if (aAllPlayesInSubGroup[yIndex].data.indexOf('Invisible="1"') == -1)
								fInvisibile=false;
						}
						
						if (!fInvisibile) {
							aGroups[iIndex].members = arrayMembers;
							arrayGroups.push(aGroups[iIndex]);
						}
					}
				}
				return arrayGroups;
			}			
		} catch (e) {
			lWarn.logln("Error executing upnpGetSonosGroups command " + e);
		}
	}
	return null;
}

function upnpGetSonosGrouping(szIpAddressRenderers, oCurlOptions) {
	var http_headers = [];
	http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
	http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:ZoneGroupTopology:1#GetZoneGroupState"';//

	var h = new HTTP(oCurlOptions, http_headers);

	for ( var kIndex = 0; kIndex < szIpAddressRenderers.length; kIndex++) {
		try {
			h.reset();
			var szIpAddressRenderer = szIpAddressRenderers[kIndex];
	
			var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/ZoneGroupTopology/Control';
	
			var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
					+ '<s:Body>' + ' <u:GetZoneGroupState xmlns:u="urn:schemas-upnp-org:service:ZoneGroupTopology:1" />' + '</s:Body>' + '</s:Envelope>';
			var result = h.post(url, '', action);
			var arrayGroups = new Array();
			if (result != null && result.body != null) {
				var aGroups = getXMLFields(result.body, 'ZoneGroup', '');
				if (aGroups != null && aGroups.length > 0) {
					for ( var iIndex = 0; iIndex < aGroups.length; iIndex++) {	
						var fInvisibile=true;
						var oResultGroup={};
						oResultGroup.id=aGroups[iIndex].data.match(/ID=\"(.*?)\"/)[1];
						oResultGroup.master=aGroups[iIndex].data.match(/Coordinator=\"(.*?)\"/)[1];
						oResultGroup.members=[];
						var aAllPlayesInGroup = getXMLFields(aGroups[iIndex].data, 'ZoneGroupMember', '/>');
						for ( var jIndex = 0; jIndex < aAllPlayesInGroup.length; jIndex++) {							
							if (aAllPlayesInGroup[jIndex].data.indexOf('Invisible="1"') == -1)
								fInvisibile=false;
						}
						if (!fInvisibile)
							for ( var jIndex = 0; jIndex < aAllPlayesInGroup.length; jIndex++) {														
								var oSlaveObject={};
								oSlaveObject.id=aAllPlayesInGroup[jIndex].data.match(/UUID=\"(.*?)\"/)[1];
								oSlaveObject.ip=aAllPlayesInGroup[jIndex].data.match(/Location=\"http:\/\/(.*?):/)[1];
								if (oResultGroup.id==oSlaveObject.id)
									oResultGroup.ip=aAllPlayesInGroup[jIndex].data.match(/Location=\"http:\/\/(.*?):/)[1];
								oResultGroup.members.push(oSlaveObject);
							}
							arrayGroups.push(oResultGroup);
					}
				}
			}
			return arrayGroups;
		} catch (e) {
			lWarn.logln("Error executing upnpGetSonosGrouping command " + e);
		}
	}
	return null;
}

function upnpGetSonosGroupID(szIpAddressRenderer, sDeviceID, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:ZoneGroupTopology:1#GetZoneGroupState"';//

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();
		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/ZoneGroupTopology/Control';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + ' <u:GetZoneGroupState xmlns:u="urn:schemas-upnp-org:service:ZoneGroupTopology:1" />' + '</s:Body>' + '</s:Envelope>';
		var result = h.post(url, '', action);
		var sonosGroupID = "";
		if (result != null && result.body != null) {
			var aGroups = getXMLFields(result.body, 'ZoneGroup', '</ZoneGroup>');
			if (aGroups != null && aGroups.length > 0) {
				for ( var iIndex = 0; iIndex < aGroups.length; iIndex++) {
					if (aGroups[iIndex].data.indexOf('Invisible="1"') == -1 && aGroups[iIndex].data.indexOf(sDeviceID) > -1) {
						sonosGroupID = aGroups[iIndex].data.substring(aGroups[iIndex].data.indexOf('ID="') + 4, aGroups[iIndex].data.indexOf('">'));
					}
				}
			}
		}
		return sonosGroupID;
	} catch (e) {
		lWarn.logln("Error executing upnpGetSonosGroupID command " + e);
	}
	return "";
}

function upnpSaveQueue(szIpAddressRenderer, sDeviceID, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#SaveQueue"';//

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();
		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';
		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + ' <u:SaveQueue xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '   <InstanceID>0</InstanceID>' + '   <Title>' + TMP_SONOS_QUEUE_NAME + '-' + sDeviceID
				+ '</Title>' + '   <ObjectID />' + ' </u:SaveQueue>' + '</s:Body>' + '</s:Envelope>';
		var result = h.post(url, '', action);
		var sQueueID = "";
		if (result != null && result.body != null) {
			matches = result.body.match(/<AssignedObjectID>(.*?)<\/AssignedObjectID>/);
			if (matches != null)
				sQueueID = matches[1];
		}
		return sQueueID;
	} catch (e) {
		lWarn.logln("Error executing upnpSaveQueue command " + e);
	}
	return "";
}

function upnpDeleteQueueFromList(szIpAddressRenderer, sQueueID, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:ContentDirectory:1#DestroyObject"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaServer/ContentDirectory/Control';
		lInfo.logln("URL " + url);

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + ' <u:DestroyObject xmlns:u="urn:schemas-upnp-org:service:ContentDirectory:1">' + '   <ObjectID>' + sQueueID + '</ObjectID>' + ' </u:DestroyObject>' + '</s:Envelope>';

		h.post(url, '', action);
	} catch (e) {
		lInfo.logln("Error executing upnpDeleteQueueFromList command " + e);
	}
}

function upnpSetPlayerAsStandAloneGroup(szIpAddressRenderer, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:AVTransport:1#BecomeCoordinatorOfStandaloneGroup"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + SONOS_PORT + '/MediaRenderer/AVTransport/Control';
		lInfo.logln("URL " + url);

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + ' <u:BecomeCoordinatorOfStandaloneGroup xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '   <InstanceID>0</InstanceID>'
				+ ' </u:BecomeCoordinatorOfStandaloneGroup>' + '</s:Envelope>';

		h.post(url, '', action);
	} catch (e) {
		lInfo.logln("Error executing upnpSetPlayerAsStandAloneGroup command " + e);
	}
}

/*
 * function upnpGetGroupCoordinator(szIpAddressRenderer, sDeviceID,
 * oCurlOptions) { try { var http_headers = []; http_headers['extra_hrd1'] =
 * 'Content-Type: text/xml; charset="utf-8"'; http_headers['extra_hdr2'] =
 * 'SOAPACTION:
 * "urn:schemas-upnp-org:service:ZoneGroupTopology:1#GetZoneGroupAttributes"';//
 * 
 * var h = new HTTP(oCurlOptions, http_headers);
 * 
 * h.reset(); var url = 'http://'+ szIpAddressRenderer + ':' + SONOS_PORT +
 * '/ZoneGroupTopology/Control';
 * 
 * var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope
 * s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
 * xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">' + '<s:Body>' + '
 * <u:GetZoneGroupAttributes
 * xmlns:u="urn:schemas-upnp-org:service:ZoneGroupTopology:1" />' + '</s:Body>' + '</s:Envelope>';
 * result = h.post(url, '', action); if (result.body != null) { var sData =
 * upnpParseXMLRespond(result.body, 'CurrentZonePlayerUUIDsInGroup', ''); var
 * sCoordinator = ""; if (sData.indexOf(',')>-1) { // var aPlayers =
 * sData.split(","); if (aPlayers.length > 0) { if
 * (aPlayers[0].indexOf(sDeviceID)==-1) { // coordinator of group is not current
 * player sCoordinator = aPlayers[0]; } } } return sCoordinator; } } catch (e) {
 * lWarn.logln("Error executing upnpGetGroupCoordinator command " + e); } return
 * ""; }
 */
